@@ -66,13 +66,14 @@ def x() -> Optional[int]:
 
 ... And a bunch more. Maybe. Tbh this is mostly for fun."""
 
-from typing import Any, Callable, Optional, ParamSpec, TypeVar, overload
+from asyncio import iscoroutinefunction
+from typing import Any, Callable, Coroutine, Optional, TypeVar, overload
 
 T = TypeVar("T")
 
 
 @overload
-def iife(c: None = None, /, **kwargs) -> Callable[[Callable[..., T]], T]:
+def iife(c=None, /, **kwargs) -> Callable[[Callable[..., T]], T]:
     ...
 
 
@@ -81,16 +82,31 @@ def iife(c: Callable[[], T], /, **kwargs) -> T:
     ...
 
 
+# TODO: async functions
+# Should async IIFEs evaluate to the coroutine or run the coroutine and await the result?
+@overload
+def iife(c: Callable[[], Coroutine[Any, Any, T]], /, **kwargs) -> T:
+    ...
+
+
 def iife(
-    c: Optional[Callable[[], T]] = None, /, **kwargs
-) -> T | Callable[[Callable[..., T]], T]:
+    c: Optional[Callable[[], T | Coroutine[Any, Any, T]]] = None, /, **kwargs
+) -> T | Callable[[Callable[..., T | Coroutine[Any, Any, T]]], T]:
     """Call the function/class and assign the result to the name of the function/class."""
+    if iscoroutinefunction(c):
+        raise NotImplementedError("iife cannot be used with async functions yet")
+
     if isinstance(c, Callable) and not kwargs:
         return c()
 
     else:
 
-        def inner(c: Callable[..., T]) -> T:
-            return c(**kwargs)
+        def inner(c: Callable[..., T | Coroutine[Any, Any, T]]) -> T:
+            if iscoroutinefunction(c):
+                raise NotImplementedError(
+                    "iife cannot be used with async functions yet"
+                )
+            else:
+                return c(**kwargs)  # type: ignore
 
         return inner
